# -*- coding: utf-8 -*-
from django.utils.translation import gettext_lazy as _
from rest_framework import exceptions, viewsets
from rest_framework.decorators import action
from rest_framework.generics import get_object_or_404
from rest_framework.response import Response
from thx.api.mixins import ThuxModelMixin

from ... import models
from . import serializers

STATUS_CODES = ((0, _('disable')), (1, _('enable')),)


# TOUR
class TourViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    TourViewSet
    -----------

    :action list: Return the tour list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given tour.
        `Profiles`: `superuser`, `staff`

    :action light: Return the tour list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new tour instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a tour instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a tour instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a tour instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.Tour.objects.all().prefetch_related("boxes")
    serializer_class = serializers.TourSerializer
    lookup_fields = ['key', 'code']

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            return
        if user.is_staff and profile == 'staff':
            return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'Tour{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    def get_object_by_code(self, code, language):
        queryset = self.filter_queryset(self.get_queryset())
        filter_kwargs = {'code': code, 'language': language}
        obj = get_object_or_404(queryset, **filter_kwargs)
        self.check_object_permissions(self.request, obj)
        return obj

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a tour instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a tour instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)

    @action(methods=['get'], detail=False, url_path=r'(?P<code>[-\w]+)/(?P<language>\w+)/get_by_code')
    def get_by_code(self, request, **filters):
        code = self.kwargs.get('code')
        language = self.kwargs.get('language')
        instance = self.get_object_by_code(code, language)
        serializer = self.get_serializer(instance)
        return Response(serializer.data)


# BOX
class BoxViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    BoxViewSet
    -----------

    :action list: Return the box list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given box.
        `Profiles`: `superuser`, `staff`

    :action light: Return the box list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new box instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a box instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a box instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a box instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.Box.objects.all()
    serializer_class = serializers.BoxSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            return
        if user.is_staff and profile == 'staff':
            return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'Box{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a box instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a box instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)
